@@ -29,8 +29,8 @@
  */
 
 #ifndef lint
-static const char rcsid[] =
-    "@(#) $Header: /tcpdump/master/libpcap/savefile.c,v 1.55.2.7 2002/06/07 04:31:56 guy Exp $ (LBL)";
+static const char rcsid[] _U_ =
+    "@(#) $Header: /tcpdump/master/libpcap/savefile.c,v 1.92.2.8 2003/12/21 21:52:36 guy Exp $ (LBL)";
 #endif
 
 #ifdef HAVE_CONFIG_H
@@ -57,7 +57,10 @@ static const char rcsid[] =
  * because time is at a premium when we are writing the file.
  * In other words, the pcap_file_header and pcap_pkthdr,
  * records are written in host byte order.
- * Note that the packets are always written in network byte order.
+ * Note that the bytes of packet data are written out in the order in
+ * which they were received, so multi-byte fields in packets are not
+ * written in host byte order, they're written in whatever order the
+ * sending machine put them in.
  *
  * ntoh[ls] aren't sufficient because we might need to swap on a big-endian
  * machine (if the file was written in little-end order).
@@ -123,7 +126,7 @@ static const char rcsid[] =
 #define LINKTYPE_PRONET		DLT_PRONET
 #define LINKTYPE_CHAOS		DLT_CHAOS
 #define LINKTYPE_TOKEN_RING	DLT_IEEE802	/* DLT_IEEE802 is used for Token Ring */
-#define LINKTYPE_ARCNET		DLT_ARCNET
+#define LINKTYPE_ARCNET		DLT_ARCNET	/* BSD-style headers */
 #define LINKTYPE_SLIP		DLT_SLIP
 #define LINKTYPE_PPP		DLT_PPP
 #define LINKTYPE_FDDI		DLT_FDDI
@@ -154,29 +157,59 @@ static const char rcsid[] =
 #define LINKTYPE_C_HDLC		104		/* Cisco HDLC */
 #define LINKTYPE_IEEE802_11	105		/* IEEE 802.11 (wireless) */
 #define LINKTYPE_ATM_CLIP	106		/* Linux Classical IP over ATM */
+#define LINKTYPE_FRELAY		107		/* Frame Relay */
 #define LINKTYPE_LOOP		108		/* OpenBSD loopback */
+#define LINKTYPE_ENC		109		/* OpenBSD IPSEC enc */
 
 #define LINKTYPE_LINUX_SLL	113		/* Linux cooked socket capture */
 #define LINKTYPE_LTALK		114		/* Apple LocalTalk hardware */
 #define LINKTYPE_ECONET		115		/* Acorn Econet */
 
+#define LINKTYPE_PFLOG		117		/* OpenBSD DLT_PFLOG */
 #define LINKTYPE_CISCO_IOS	118		/* For Cisco-internal use */
 #define LINKTYPE_PRISM_HEADER	119		/* 802.11+Prism II monitor mode */
 #define LINKTYPE_AIRONET_HEADER	120		/* FreeBSD Aironet driver stuff */
+#define LINKTYPE_IP_OVER_FC	122		/* RFC 2625 IP-over-Fibre Channel */
+#define LINKTYPE_SUNATM		123		/* Solaris+SunATM */
+
+#define LINKTYPE_IEEE802_11_RADIO 127		/* 802.11 plus WLAN header */
+
+#define LINKTYPE_TZSP		128		/* Tazmen Sniffer Protocol */
+
+#define LINKTYPE_ARCNET_LINUX	129		/* Linux-style headers */
+
+#define LINKTYPE_JUNIPER_MLPPP  130		/* Juniper-internal chassis encapsulation */
+#define LINKTYPE_JUNIPER_MLFR   131
+#define LINKTYPE_JUNIPER_ES     132
+#define LINKTYPE_JUNIPER_GGSN   133
+#define LINKTYPE_JUNIPER_MFR    134
+#define LINKTYPE_JUNIPER_ATM2   135
+#define LINKTYPE_JUNIPER_SERVICES 136
+#define LINKTYPE_JUNIPER_ATM1   137
+
+#define LINKTYPE_APPLE_IP_OVER_IEEE1394 138	/* Apple IP-over-IEEE 1394 cooked header */
+
+#define LINKTYPE_RAWSS7         139             /* see rawss7.h for */
+#define LINKTYPE_RAWSS7_MTP2    140	        /* information  on these */
+#define LINKTYPE_RAWSS7_MTP3    141             /* definitions */
+#define LINKTYPE_RAWSS7_SCCP    142
+
+#define LINKTYPE_LINUX_IRDA	144		/* Linux-IrDA */
 
 /*
  * These types are reserved for future use.
  */
-#define LINKTYPE_FRELAY		107		/* Frame Relay */
-#define LINKTYPE_ENC		109		/* OpenBSD IPSEC enc */
 #define LINKTYPE_LANE8023	110		/* ATM LANE + 802.3 */
 #define LINKTYPE_HIPPI		111		/* NetBSD HIPPI */
 #define LINKTYPE_HDLC		112		/* NetBSD HDLC framing */
 #define LINKTYPE_IPFILTER	116		/* IP Filter capture files */
-#define LINKTYPE_PFLOG		117		/* OpenBSD DLT_PFLOG */
 #define LINKTYPE_HHDLC		121		/* Siemens HiPath HDLC */
-#define LINKTYPE_IP_OVER_FC	122		/* RFC 2625 IP-over-Fibre Channel */
-#define LINKTYPE_SUNATM		123		/* Solaris+SunATM */
+#define LINKTYPE_RIO		124		/* RapidIO */
+#define LINKTYPE_PCI_EXP	125		/* PCI Express */
+#define LINKTYPE_AURORA		126		/* Xilinx Aurora link layer */
+#define LINKTYPE_DOCSIS		143		/* DOCSIS MAC frames */
+#define LINKTYPE_IBM_SP		145		/* IBM SP switch */
+#define LINKTYPE_IBM_SN		146		/* IBM Next Federation switch */
 
 static struct linktype_map {
 	int	dlt;
@@ -250,6 +283,9 @@ static struct linktype_map {
 	/* Acorn Econet */
 	{ DLT_ECONET,		LINKTYPE_ECONET },
 
+	/* OpenBSD DLT_PFLOG */
+	{ DLT_PFLOG,		LINKTYPE_PFLOG },
+
 	/* For Cisco-internal use */
 	{ DLT_CISCO_IOS,	LINKTYPE_CISCO_IOS },
 
@@ -268,6 +304,47 @@ static struct linktype_map {
 	/* Solaris+SunATM */
 	{ DLT_SUNATM,		LINKTYPE_SUNATM },
 
+	/* RapidIO */
+	{ DLT_RIO,		LINKTYPE_RIO },
+
+	/* PCI Express */
+	{ DLT_PCI_EXP,		LINKTYPE_PCI_EXP },
+
+	/* Xilinx Aurora link layer */
+	{ DLT_AURORA,		LINKTYPE_AURORA },
+
+	/* 802.11 plus WLAN header */
+	{ DLT_IEEE802_11_RADIO,	LINKTYPE_IEEE802_11_RADIO },
+
+	/* Tazmen Sniffer Protocol */
+	{ DLT_TZSP,		LINKTYPE_TZSP },
+
+	/* Arcnet with Linux-style link-layer headers */
+	{ DLT_ARCNET_LINUX,	LINKTYPE_ARCNET_LINUX },
+
+        /* Juniper-internal chassis encapsulation */
+        { DLT_JUNIPER_MLPPP,    LINKTYPE_JUNIPER_MLPPP },
+        { DLT_JUNIPER_MLFR,     LINKTYPE_JUNIPER_MLFR },
+        { DLT_JUNIPER_ES,       LINKTYPE_JUNIPER_ES },
+        { DLT_JUNIPER_GGSN,     LINKTYPE_JUNIPER_GGSN },
+        { DLT_JUNIPER_MFR,      LINKTYPE_JUNIPER_MFR },
+        { DLT_JUNIPER_ATM2,     LINKTYPE_JUNIPER_ATM2 },
+        { DLT_JUNIPER_SERVICES, LINKTYPE_JUNIPER_SERVICES },
+        { DLT_JUNIPER_ATM1,     LINKTYPE_JUNIPER_ATM1 },
+
+	/* Apple IP-over-IEEE 1394 cooked header */
+	{ DLT_APPLE_IP_OVER_IEEE1394, LINKTYPE_APPLE_IP_OVER_IEEE1394 },
+
+	/* DOCSIS MAC frames */
+	{ DLT_DOCSIS,		LINKTYPE_DOCSIS },
+
+	/* IrDA IrLAP packets + Linux-cooked header */
+	{ DLT_LINUX_IRDA,	LINKTYPE_LINUX_IRDA },
+
+	/* IBM SP and Next Federation switches */
+	{ DLT_IBM_SP,		LINKTYPE_IBM_SP },
+	{ DLT_IBM_SN,		LINKTYPE_IBM_SN },
+
 	/*
 	 * Any platform that defines additional DLT_* codes should:
 	 *
@@ -356,6 +433,43 @@ swap_hdr(struct pcap_file_header *hp)
 	hp->linktype = SWAPLONG(hp->linktype);
 }
 
+static int
+sf_getnonblock(pcap_t *p, char *errbuf)
+{
+	/*
+	 * This is a savefile, not a live capture file, so never say
+	 * it's in non-blocking mode.
+	 */
+	return (0);
+}
+
+static int
+sf_setnonblock(pcap_t *p, int nonblock, char *errbuf)
+{
+	/*
+	 * This is a savefile, not a live capture file, so ignore
+	 * requests to put it in non-blocking mode.
+	 */
+	return (0);
+}
+
+static int
+sf_stats(pcap_t *p, struct pcap_stat *ps)
+{
+	snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+	    "Statistics aren't available from savefiles");
+	return (-1);
+}
+
+static void
+sf_close(pcap_t *p)
+{
+	if (p->sf.rfile != stdin)
+		(void)fclose(p->sf.rfile);
+	if (p->sf.base != NULL)
+		free(p->sf.base);
+}
+
 pcap_t *
 pcap_open_offline(const char *fname, char *errbuf)
 {
@@ -372,15 +486,15 @@ pcap_open_offline(const char *fname, char *errbuf)
 	}
 
 	memset((char *)p, 0, sizeof(*p));
-	/*
-	 * Set this field so we don't close stdin in pcap_close!
-	 */
-	p->fd = -1;
 
 	if (fname[0] == '-' && fname[1] == '\0')
 		fp = stdin;
 	else {
+#ifndef WIN32
 		fp = fopen(fname, "r");
+#else
+		fp = fopen(fname, "rb");
+#endif
 		if (fp == NULL) {
 			snprintf(errbuf, PCAP_ERRBUF_SIZE, "%s: %s", fname,
 			    pcap_strerror(errno));
@@ -421,7 +535,12 @@ pcap_open_offline(const char *fname, char *errbuf)
 	p->snapshot = hdr.snaplen;
 	p->linktype = linktype_to_dlt(hdr.linktype);
 	p->sf.rfile = fp;
+#ifndef WIN32
 	p->bufsize = hdr.snaplen;
+#else
+	/* Allocate the space for pcap_pkthdr as well. It will be used by pcap_read_ex */
+	p->bufsize = hdr.snaplen+sizeof(struct pcap_pkthdr);
+#endif
 
 	/* Align link header as required for proper data alignment */
 	/* XXX should handle all types */
@@ -456,8 +575,59 @@ pcap_open_offline(const char *fname, char *errbuf)
 	pcap_fddipad = 0;
 #endif
 
+	/*
+	 * We interchanged the caplen and len fields at version 2.3,
+	 * in order to match the bpf header layout.  But unfortunately
+	 * some files were written with version 2.3 in their headers
+	 * but without the interchanged fields.
+	 *
+	 * In addition, DG/UX tcpdump writes out files with a version
+	 * number of 543.0, and with the caplen and len fields in the
+	 * pre-2.3 order.
+	 */
+	switch (hdr.version_major) {
+
+	case 2:
+		if (hdr.version_minor < 3)
+			p->sf.lengths_swapped = SWAPPED;
+		else if (hdr.version_minor == 3)
+			p->sf.lengths_swapped = MAYBE_SWAPPED;
+		else
+			p->sf.lengths_swapped = NOT_SWAPPED;
+		break;
+
+	case 543:
+		p->sf.lengths_swapped = SWAPPED;
+		break;
+
+	default:
+		p->sf.lengths_swapped = NOT_SWAPPED;
+		break;
+	}
+
+#ifndef WIN32
+	/*
+	 * You can do "select()" and "poll()" on plain files on most
+	 * platforms, and should be able to do so on pipes.
+	 *
+	 * You can't do "select()" on anything other than sockets in
+	 * Windows, so, on Win32 systems, we don't have "selectable_fd".
+	 */
+	p->selectable_fd = fileno(fp);
+#endif
+
+	p->read_op = pcap_offline_read;
+	p->setfilter_op = install_bpf_program;
+	p->set_datalink_op = NULL;	/* we don't support munging link-layer headers */
+	p->getnonblock_op = sf_getnonblock;
+	p->setnonblock_op = sf_setnonblock;
+	p->stats_op = sf_stats;
+	p->close_op = sf_close;
+
 	return (p);
  bad:
+	if(fp)
+		fclose(fp);
 	free(p);
 	return (NULL);
 }
@@ -468,10 +638,12 @@ pcap_open_offline(const char *fname, char *errbuf)
  * no more packets, and SFERR_TRUNC if a partial packet was encountered.
  */
 static int
-sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
+sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, u_int buflen)
 {
 	struct pcap_sf_patched_pkthdr sf_hdr;
 	FILE *fp = p->sf.rfile;
+	size_t amt_read;
+	bpf_u_int32 t;
 
 	/*
 	 * Read the packet header; the structure we use as a buffer
@@ -480,9 +652,23 @@ sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
 	 * unpatched libpcap we only read as many bytes as the regular
 	 * header has.
 	 */
-	if (fread(&sf_hdr, p->sf.hdrsize, 1, fp) != 1) {
-		/* probably an EOF, though could be a truncated packet */
-		return (1);
+	amt_read = fread(&sf_hdr, 1, p->sf.hdrsize, fp);
+	if (amt_read != p->sf.hdrsize) {
+		if (ferror(fp)) {
+			snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+			    "error reading dump file: %s",
+			    pcap_strerror(errno));
+			return (-1);
+		} else {
+			if (amt_read != 0) {
+				snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+				    "truncated dump file; tried to read %d header bytes, only got %lu",
+				    p->sf.hdrsize, (unsigned long)amt_read);
+				return (-1);
+			}
+			/* EOF */
+			return (1);
+		}
 	}
 
 	if (p->sf.swapped) {
@@ -497,17 +683,27 @@ sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
 		hdr->ts.tv_sec = sf_hdr.ts.tv_sec;
 		hdr->ts.tv_usec = sf_hdr.ts.tv_usec;
 	}
-	/*
-	 * We interchanged the caplen and len fields at version 2.3,
-	 * in order to match the bpf header layout.  But unfortunately
-	 * some files were written with version 2.3 in their headers
-	 * but without the interchanged fields.
-	 */
-	if (p->sf.version_minor < 3 ||
-	    (p->sf.version_minor == 3 && hdr->caplen > hdr->len)) {
-		int t = hdr->caplen;
+	/* Swap the caplen and len fields, if necessary. */
+	switch (p->sf.lengths_swapped) {
+
+	case NOT_SWAPPED:
+		break;
+
+	case MAYBE_SWAPPED:
+		if (hdr->caplen <= hdr->len) {
+			/*
+			 * The captured length is <= the actual length,
+			 * so presumably they weren't swapped.
+			 */
+			break;
+		}
+		/* FALLTHROUGH */
+
+	case SWAPPED:
+		t = hdr->caplen;
 		hdr->caplen = hdr->len;
 		hdr->len = t;
+		break;
 	}
 
 	if (hdr->caplen > buflen) {
@@ -518,7 +714,7 @@ sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
 		 * grossly wrong, try to salvage.
 		 */
 		static u_char *tp = NULL;
-		static int tsize = 0;
+		static size_t tsize = 0;
 
 		if (hdr->caplen > 65535) {
 			snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
@@ -538,9 +734,17 @@ sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
 				return (-1);
 			}
 		}
-		if (fread((char *)tp, hdr->caplen, 1, fp) != 1) {
-			snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
-			    "truncated dump file");
+		amt_read = fread((char *)tp, 1, hdr->caplen, fp);
+		if (amt_read != hdr->caplen) {
+			if (ferror(fp)) {
+				snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+				    "error reading dump file: %s",
+				    pcap_strerror(errno));
+			} else {
+				snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+				    "truncated dump file; tried to read %u captured bytes, only got %lu",
+				    hdr->caplen, (unsigned long)amt_read);
+			}
 			return (-1);
 		}
 		/*
@@ -555,10 +759,17 @@ sf_next_packet(pcap_t *p, struct pcap_pkthdr *hdr, u_char *buf, int buflen)
 
 	} else {
 		/* read the packet itself */
-
-		if (fread((char *)buf, hdr->caplen, 1, fp) != 1) {
-			snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
-			    "truncated dump file");
+		amt_read = fread((char *)buf, 1, hdr->caplen, fp);
+		if (amt_read != hdr->caplen) {
+			if (ferror(fp)) {
+				snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+				    "error reading dump file: %s",
+				    pcap_strerror(errno));
+			} else {
+				snprintf(p->errbuf, PCAP_ERRBUF_SIZE,
+				    "truncated dump file; tried to read %u captured bytes, only got %lu",
+				    hdr->caplen, (unsigned long)amt_read);
+			}
 			return (-1);
 		}
 	}
@@ -579,6 +790,23 @@ pcap_offline_read(pcap_t *p, int cnt, pcap_handler callback, u_char *user)
 	while (status == 0) {
 		struct pcap_pkthdr h;
 
+		/*
+		 * Has "pcap_breakloop()" been called?
+		 * If so, return immediately - if we haven't read any
+		 * packets, clear the flag and return -2 to indicate
+		 * that we were told to break out of the loop, otherwise
+		 * leave the flag set, so that the *next* call will break
+		 * out of the loop without having read any packets, and
+		 * return the number of packets we've processed so far.
+		 */
+		if (p->break_loop) {
+			if (n == 0) {
+				p->break_loop = 0;
+				return (-2);
+			} else
+				return (n);
+		}
+
 		status = sf_next_packet(p, &h, p->buffer, p->bufsize);
 		if (status) {
 			if (status == 1)
@@ -633,10 +861,18 @@ pcap_dump_open(pcap_t *p, const char *fname)
 		return (NULL);
 	}
 
-	if (fname[0] == '-' && fname[1] == '\0')
+	if (fname[0] == '-' && fname[1] == '\0') {
 		f = stdout;
-	else {
+#ifdef WIN32
+		_setmode(_fileno(f), _O_BINARY);
+#endif
+	} else {
+#ifndef WIN32
 		f = fopen(fname, "w");
+#else
+		f = fopen(fname, "wb");
+		setbuf(f, NULL);	/* XXX - why? */
+#endif
 		if (f == NULL) {
 			snprintf(p->errbuf, PCAP_ERRBUF_SIZE, "%s: %s",
 			    fname, pcap_strerror(errno));
@@ -647,6 +883,22 @@ pcap_dump_open(pcap_t *p, const char *fname)
 	return ((pcap_dumper_t *)f);
 }
 
+FILE *
+pcap_dump_file(pcap_dumper_t *p)
+{
+	return ((FILE *)p);
+}
+
+int
+pcap_dump_flush(pcap_dumper_t *p)
+{
+
+	if (fflush((FILE *)p) == EOF)
+		return (-1);
+	else
+		return (0);
+}
+
 void
 pcap_dump_close(pcap_dumper_t *p)
 {
